#include "asmegam.h"
#include "lcd.h"

void port_init(void)
{
	DDRD |= (LED0 | LED1);
	PORTD |= (LED0 | LED1);

	PORTC = 0;
}

void timer0_init(void)
{
	TCCR0 = 0x00;
	ASSR  = 0x08;
	TCNT0 = 0x00;
	TCCR0 = 0x04;
}

// hw mode

void mem_init()
{
	PORTA = 0x00;
	PORTC = 0x00;
	PORTG = 3; // rd|wr
	DDRA = 0xff;
	DDRC = 0xff;
	DDRG |= 7;

	MCUCR |= (1<<7) | (1<<6); // sram enable, sram wait state
	XMCRA = (1<<3) | (1<<2); // all lower section, max. waitstates
	XMCRB = (1<<7) | (1<<0); // bus keeper + release pc7
}

void mem_deinit()
{
	MCUCR &= ~((1<<7) | (1<<6)); // sram enable, sram wait state
	XMCRA &= ~((1<<3) | (1<<2)); // all lower section, max. waitstates
	XMCRB &= ~((1<<7) | (1<<0)); // bus keeper + release pc7
}

volatile byte *mem = ((volatile unsigned char *)0x0000);

void mem_write(word a, byte d)
{
	mem[a] = d;
}

byte mem_read(word a)
{
	return mem[a];
}

byte hex[16] = "0123456789abcdef";

void puthexbyte(byte a)
{
	putchar(hex[a>>4]);
	putchar(hex[a&15]);
}

void puthexword(word a)
{
	putchar(hex[(a>>12)%15]);
	putchar(hex[(a>>8)%15]);
	putchar(hex[(a>>4)%15]);
	putchar(hex[a&15]);
}

byte f(word i, byte t)
{
	if(t == 0)
		return 0;
	else if(t == 1)
		return 0xff;
	else 
		return (i>>(t-2))&0xff;
}

void main(void) //  
{
	word i,t;
	byte ok;

	CLI(); //   
	XDIV  = 0x00; //  . ICCAVR AppBuilder
	XMCRA = 0x0A; //external memory
	XMCRB = 0x00;
	port_init();
	MCUCR = 0x00;
	EICRA = 0x00;
	EICRB = 0x00;
	EIMSK = 0x00;
	TIMSK = 0x01;
	ETIMSK = 0x00;
		
	init_lcd(); //    

	gotoxy(0,0);
	outtext("Testing...");

	gotoxy(0,1);
	outtext("Pass ");

	ok = 1;

	for(t = 0; t < 8 && ok; t++)
	{
		putchar(t+'A');

		mem_init();
	
		for(i = 0x1100; i < 0x8000; i++)
			mem_write(i,f(i,t));

		for(i = 0x1100; i < 0x8000 && ok; i++)
			if(mem_read(i) != f(i,t))
				ok = 0;
	
		mem_deinit();
	}

	clrscr();

	if(ok)
		outtext("SRAM present");
	else
	{
		outtext("SRAM not present");
		gotoxy(0,1);
		outtext("Test=");
		putchar(t+'A');
		outtext(" Addr=");
		puthexword(i);
	}
}
