/* by Termostat  avr123.nm.ru  or  avr123.by.ru   2004_12
 

      // AT45d161 to SPI Port connection
      
      // SCK    SO      SI      CS             WP    RDY/BSY    RST                 
      // PB7    PB6     PB5     PB4     PB3    PB2     PB1      PB0 
      
      //                
      // Out    In      Out     Out     In     Out     In       Out                  
      //  1      0       1       1       1      1       0        1          
      
    PB1 - RDY      0x02
    
    PB4 - CS    DF_CHIP_SELECT  
    
    
    PB0 - RST     !  "1" 
    PB2 - WP     !  "1"  
    
    PB0 - RST     PB2 - WP    , 
       RST    WP     AT45 
    
*/
     

// I/O register definitions for AT90S8535
#include <90s8535.h>



// at45 DataFlash - defines for all opcodes

// DataFlash chip select port pin (PB 4)
#define DF_CHIP_SELECT 0x10

// buffer 1 write 
#define BUFFER_1_WRITE 0x84

// buffer 2 write 
#define BUFFER_2_WRITE 0x87

// buffer 1 read
#define BUFFER_1_READ 0x54

// buffer 2 read
#define BUFFER_2_READ 0x56

// buffer 1 to main memory page program with built-in erase
#define B1_TO_MM_PAGE_PROG_WITH_ERASE 0x83

// buffer 2 to main memory page program with built-in erase
#define B2_TO_MM_PAGE_PROG_WITH_ERASE 0x86

// buffer 1 to main memory page program without built-in erase
#define B1_TO_MM_PAGE_PROG_WO_ERASE 0x88

// buffer 2 to main memory page program without built-in erase
#define B2_TO_MM_PAGE_PROG_WO_ERASE 0x89

// main memory page program through buffer 1
#define MM_PAGE_PROG_THROUGH_B1 0x82
 
// main memory page program through buffer 2
#define MM_PAGE_PROG_THROUGH_B2 0x85
 
// auto page rewrite through buffer 1
#define AUTO_PAGE_REWRITE_THROUGH_B1 0x58
 
// auto page rewrite through buffer 2
#define AUTO_PAGE_REWRITE_THROUGH_B2 0x59
 
// main memory page compare to buffer 1
#define MM_PAGE_TO_B1_COMP 0x60

// main memory page compare to buffer 2
#define MM_PAGE_TO_B2_COMP 0x61
 
// main memory page to buffer 1 transfer
#define MM_PAGE_TO_B1_XFER 0x53

// main memory page to buffer 2 transfer
#define MM_PAGE_TO_B2_XFER 0x55

// DataFlash status register for reading density, compare status, 
// and ready/busy status
#define STATUS_REGISTER 0x57

// main memory page read
#define MAIN_MEMORY_PAGE_READ 0x52

// erase a 528 byte page
#define PAGE_ERASE 0x81

// erase 512 pages
#define BLOCK_ERASE 0x50


//////////////////////////////////////////////////




// Standard Input/Output functions
#include <stdio.h>

// delay functions
#include <delay.h>  

// prototypes

void erasing (void); 
//   
   
void write_to_flash (unsigned char ad_data);   
//      

void playback (void);      
//      

void next_page_to_next_buffer (unsigned char active_buffer, unsigned int page_counter); 
//       

void active_buffer_to_UART (unsigned char active_buffer);
//      



// global variables  

volatile unsigned char new_data = 0;
//  -  "1"     0- 
//  "0"         

//       
volatile unsigned char dumb_data = 0; 

//     0 -   1 - 
volatile unsigned char flash_filed = 0; 


//   ,       
unsigned int last_byte_position_buffer = 0; 

//  ,      
unsigned int last_byte_page = 0;     

     

//    . 4 
void erasing(void)
{
    unsigned int block_counter = 0;
    unsigned char temp = 0x80;
    
    new_data = 1;   
    //        0.

    // interrupt disabled, SPI port enabled, master mode, MSB first,  SPI mode 3, Fcl/4
    SPCR = 0x5C; //  SPI

    while (block_counter < 512)
    {
        PORTB &= ~DF_CHIP_SELECT;           // enable DataFlash

// putsf("TP 0001"); //  8    
//  ,   
  
        SPDR = BLOCK_ERASE;
        while (!(SPSR & temp));             // wait for data transfer to be completed

// putsf("TP 0002"); //  8  
//  ,   

        SPDR = (char)(block_counter>>3); //   . 4 
        while (!(SPSR & temp));             // wait for data transfer to be completed
               
        
        SPDR = (char)(block_counter<<5); //   . 4 
        while (!(SPSR & temp));             // wait for data transfer to be completed


        SPDR = 0x00;                        // don't cares
        while (!(SPSR & temp));             // wait for data transfer to be completed
            
        
        PORTB |= DF_CHIP_SELECT;            // disable DataFlash

        block_counter++; 
        
        
        while(!(PINB & 0x02));              // wait until block is erased 
                         
    }  
    //  512   8  
    
    SPCR = 0x00;                            //disable SPI 
    //  SPI        
}

  

/////////////////////////////////////////////////////////////////////
/*
     -  :

  write_to_flash(__ );

    0  ,  
 : new_data = 1

*/     

void write_to_flash(unsigned char flash_data)
{ 

//       1.
  
// static  -      
//    !
    
    //         
    static unsigned int buffer_counter;
    
    //     
    static unsigned int page_counter;  
    
    unsigned char temp = 0x80; 
    
    // interrupt disabled, SPI port enabled, master mode
    SPCR = 0x5C; // MSB first, SPI mode 3, Fcl/4
    //  SPI
    
    //    "new_data" -   
    if(new_data) //    
    {  
        buffer_counter = 0; //    
        page_counter = 0;   //    
        new_data = 0;       //  "new_data" 
    }
                
    
    while(!(PINB & 0x02));                  // check if flash is busy
    //     - ..   RDY  "1"
    
    PORTB &= ~DF_CHIP_SELECT;               // enable DataFlash
    //  AT45db161 -  " "
    //    CS  "0"
     
    SPDR = BUFFER_1_WRITE;
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // don't cares
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = (char)(buffer_counter>>8);       // don't cares plus first two bits of buffer address
    while (!(SPSR & temp));                 // wait for data transfer to be completed        
    SPDR = (char)buffer_counter;            // buffer address (max. 2^8 = 256 pages)
    while (!(SPSR & temp));                 // wait for data transfer to be completed    
    SPDR = flash_data;                      // write data into SPI Data Register
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    
    PORTB |= DF_CHIP_SELECT;                // disable DataFlash 
       
    //        . 
    last_byte_position_buffer = buffer_counter; 

    buffer_counter++;  //          
    
        
    if (buffer_counter > 527)  //miss!!!  // if buffer full write buffer into memory page
    {  //     528 ,      
        buffer_counter = 0;
        if (page_counter < 4096)            //    ...   
        { 
            PORTB &= ~DF_CHIP_SELECT;       // enable DataFlash
        
            SPDR = B1_TO_MM_PAGE_PROG_WO_ERASE; // write data from buffer1 to page 
            while (!(SPSR & temp));         // wait for data transfer to be completed
            SPDR = (char)(page_counter>>6);
            while (!(SPSR & temp));         // wait for data transfer to be completed
            SPDR = (char)(page_counter<<2);
            while (!(SPSR & temp));         // wait for data transfer to be completed
            SPDR = 0x00;                    // don't cares
            while (!(SPSR & temp));         // wait for data transfer to be completed                      
        
            PORTB |= DF_CHIP_SELECT;        // disable DataFlash
            
            //       .  
            last_byte_page = page_counter;
            
            page_counter++;
        }
        else
        {   
        
          flash_filed = 1; //   
          
            //old PORTB |= 0x08;                  // turn LED off
            //old while (!(PIND & 2));            // wait until button for recording (button 1) is released
        }
    }
}
          








void playback(void)
{      
    unsigned int page_counter = 0;
    //not used!  unsigned int buffer_counter = 0;
    unsigned char active_buffer = 1;        
    // active buffer = buffer1     

    
    // interrupt disabled, SPI port enabled, master mode, MSB first,  SPI mode 3, Fcl/4
    SPCR = 0x5C;
        
    next_page_to_next_buffer (active_buffer, page_counter);  
    // ! read page0 to buffer1  - ! 
    //:    page_counter   (active_buffer)
    // read page0 to buffer2
        
    while (!(PINB & 0x02));                 
    //!  wait until page0 to buffer1 transaction is finished
    //: wait until page0 to buffer2 transaction is finished 
    
    
 
 //  while      , 
 //       !!!  !
   
    while (page_counter < 4095)  //      ...
    {   
        page_counter++;  // now take next page
        /*     - . .  2 */
                   
        if (active_buffer == 1)             // if buffer1 is the active buffer
        {
          active_buffer++;                    // set buffer2 as active buffer
        }
        else                                // else
        {
          active_buffer--;                    // set buffer1 as active buffer
        }
        //       -2      
        /*     - .  -1 */
           
        next_page_to_next_buffer (active_buffer, page_counter); 
        /*      . 1   1
            2- ! */
        /*     -  -2   . 2 */
        
        active_buffer_to_UART (active_buffer);
        /*       -2  . 0
            2- ! */
        /*     -  -1   . 1 */
        
    }
  
    SPCR = 0x00; // disable SPI 
    
    
}
  







void next_page_to_next_buffer (unsigned char active_buffer, unsigned int page_counter)
{
    unsigned char temp = 0x80;

    while(!(PINB & 0x02));                  // wait until flash is not busy
    
    PORTB &= ~DF_CHIP_SELECT;               // enable DataFlash
     
     
    /* :    ,    
       !!! ,   */            
    if (active_buffer == 1)                 // if buffer1 is the active buffer
    {
        SPDR = MM_PAGE_TO_B2_XFER;          // transfer next page to buffer2
    }
    else                                    // else
    {
        SPDR = MM_PAGE_TO_B1_XFER;          // transfer next page to buffer1
    }
    
    
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = (char)(page_counter >> 6);
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = (char)(page_counter << 2);
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed 
    PORTB |= DF_CHIP_SELECT;                // disable DataFlash and start transaction     
}
           
 


void active_buffer_to_UART (unsigned char active_buffer) 
{
    // until active buffer not empty read active buffer to rs232
      
    unsigned int buffer_counter = 0;
    unsigned char temp = 0x80; 
       
    
    putsf("NextPg."); //  8 
    
       
    PORTB &= ~DF_CHIP_SELECT;               // enable DataFlash     

    if (active_buffer == 1)                 // if buffer1 is the active buffer
    {
        SPDR = BUFFER_1_READ;               // read from buffer1
    }
    else                                    // else
    {
        SPDR = BUFFER_2_READ;               // read from buffer2
    }
    while (!(SPSR & temp));                 // wait for data transfer to be completed                      
    
    
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // start at buffer address 0
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed   
         
    while (buffer_counter < 528)                          
    {
        SPDR = 0xFF;                        // write dummy value to start register shift
        while (!(SPSR & temp));             // wait for data transfer to be completed
        
     
                 
        while (!USR.5); //    UDR
        
        UDR = SPDR; //     SPI    UART
        
                    
        buffer_counter++;
     }
     PORTB |= DF_CHIP_SELECT;               // disable DataFlash
}


